/*
	RT Bench - Benchmarks various hash functions for Rainbow Tables
	Copyright (C) 2012 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#define AVX_CODE
#define XOP_CODE

#include "doublemd5.h"
#include "doublemd5help.h"
#include "hex.h"

// ***********************
// **** Compile flags ****
// ***********************
// Intel: icl /QxAVX
//        icc -xAVX
// MSVS:  /arch:AVX
// GCC:   -mavx -mxop

#ifdef ARC_x86

void doubleMd5Func_XOP2(__m128i *pw, __m128i *hash)
{
	doubleMd5Func_XOP(pw, hash);
}

void DoubleMD5::calcAVXXOP(vector4 *pw, vector4 *hash)
{
	doubleMd5Func_XOP2((__m128i*) pw, (__m128i*) hash + __DOUBLE_MD5_INTERLACED__);
	if (__DOUBLE_MD5_INTERLACED__ > 0) { ((__m128i*) hash)[0 * __DOUBLE_MD5_INTERLACED__ + 0] = _mm_set1_epi32(32); }
	if (__DOUBLE_MD5_INTERLACED__ > 1) { ((__m128i*) hash)[0 * __DOUBLE_MD5_INTERLACED__ + 1] = _mm_set1_epi32(32); }
	if (__DOUBLE_MD5_INTERLACED__ > 2) { ((__m128i*) hash)[0 * __DOUBLE_MD5_INTERLACED__ + 2] = _mm_set1_epi32(32); }
	if (__DOUBLE_MD5_INTERLACED__ > 3) { ((__m128i*) hash)[0 * __DOUBLE_MD5_INTERLACED__ + 3] = _mm_set1_epi32(32); }
	if (__DOUBLE_MD5_INTERLACED__ > 0) { ((__m128i*) hash)[9 * __DOUBLE_MD5_INTERLACED__ + 0] = _mm_set1_epi32(0x80); }
	if (__DOUBLE_MD5_INTERLACED__ > 1) { ((__m128i*) hash)[9 * __DOUBLE_MD5_INTERLACED__ + 1] = _mm_set1_epi32(0x80); }
	if (__DOUBLE_MD5_INTERLACED__ > 2) { ((__m128i*) hash)[9 * __DOUBLE_MD5_INTERLACED__ + 2] = _mm_set1_epi32(0x80); }
	if (__DOUBLE_MD5_INTERLACED__ > 3) { ((__m128i*) hash)[9 * __DOUBLE_MD5_INTERLACED__ + 3] = _mm_set1_epi32(0x80); }
	__m128i vh, vl, const_0x0f, const_0x09, const_0x27, const_0x30;

	const_0x0f = _mm_set1_epi32(0x0f0f0f0f);
	const_0x09 = _mm_set1_epi32(0x09090909);
	const_0x27 = _mm_set1_epi32(0x27272727);
	const_0x30 = _mm_set1_epi32(0x30303030);

	#define x(i,j)  ((__m128i*) hash)[(i) * __DOUBLE_MD5_INTERLACED__ + (j)]
	if (__DOUBLE_MD5_INTERLACED__ > 3) { CONVERT_TO_HEX(x(4,3), x(7,3), x(8,3), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 2) { CONVERT_TO_HEX(x(4,2), x(7,2), x(8,2), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 1) { CONVERT_TO_HEX(x(4,1), x(7,1), x(8,1), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 0) { CONVERT_TO_HEX(x(4,0), x(7,0), x(8,0), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 3) { CONVERT_TO_HEX(x(3,3), x(5,3), x(6,3), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 2) { CONVERT_TO_HEX(x(3,2), x(5,2), x(6,2), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 1) { CONVERT_TO_HEX(x(3,1), x(5,1), x(6,1), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 0) { CONVERT_TO_HEX(x(3,0), x(5,0), x(6,0), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 3) { CONVERT_TO_HEX(x(2,3), x(3,3), x(4,3), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 2) { CONVERT_TO_HEX(x(2,2), x(3,2), x(4,2), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 1) { CONVERT_TO_HEX(x(2,1), x(3,1), x(4,1), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 0) { CONVERT_TO_HEX(x(2,0), x(3,0), x(4,0), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 3) { CONVERT_TO_HEX(x(1,3), x(1,3), x(2,3), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 2) { CONVERT_TO_HEX(x(1,2), x(1,2), x(2,2), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 1) { CONVERT_TO_HEX(x(1,1), x(1,1), x(2,1), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	if (__DOUBLE_MD5_INTERLACED__ > 0) { CONVERT_TO_HEX(x(1,0), x(1,0), x(2,0), vl, vh, const_0x0f, const_0x09, const_0x27, const_0x30); }
	#undef x
	doubleMd5Func_XOP2((__m128i*) hash, (__m128i*) hash);
}

#endif
