/*
	RT Bench - Benchmarks various hash functions for Rainbow Tables
	Copyright (C) 2012 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "hashfactory.h"
#include "mysqlsha1.h"
#include "sha1.h"
#include "sha1help.h"

uint32_t MySQLSHA1::inited = MySQLSHA1::initCalc();

#ifdef ARC_x86

void MySQLSHA1::calcSSSE3(vector4 *pw, vector4 *hash)
{
	SHA1::calcSSSE3(pw, hash + __SHA1_INTERLACED__);
	if (__SHA1_INTERLACED__ > 0) { ((__m128i*) hash)[0 * __SHA1_INTERLACED__ + 0] = _mm_set1_epi32(20); }
	if (__SHA1_INTERLACED__ > 1) { ((__m128i*) hash)[0 * __SHA1_INTERLACED__ + 1] = _mm_set1_epi32(20); }
	if (__SHA1_INTERLACED__ > 2) { ((__m128i*) hash)[0 * __SHA1_INTERLACED__ + 2] = _mm_set1_epi32(20); }
	if (__SHA1_INTERLACED__ > 0) { ((__m128i*) hash)[6 * __SHA1_INTERLACED__ + 0] = _mm_set1_epi32(0x80); }
	if (__SHA1_INTERLACED__ > 1) { ((__m128i*) hash)[6 * __SHA1_INTERLACED__ + 1] = _mm_set1_epi32(0x80); }
	if (__SHA1_INTERLACED__ > 2) { ((__m128i*) hash)[6 * __SHA1_INTERLACED__ + 2] = _mm_set1_epi32(0x80); }
	SHA1::calcSSSE3(hash, hash);
}

void MySQLSHA1::calcSSE2(vector4 *pw, vector4 *hash)
{
	SHA1::calcSSE2(pw, hash + __SHA1_INTERLACED__);
	if (__SHA1_INTERLACED__ > 0) { ((__m128i*) hash)[0 * __SHA1_INTERLACED__ + 0] = _mm_set1_epi32(20); }
	if (__SHA1_INTERLACED__ > 1) { ((__m128i*) hash)[0 * __SHA1_INTERLACED__ + 1] = _mm_set1_epi32(20); }
	if (__SHA1_INTERLACED__ > 2) { ((__m128i*) hash)[0 * __SHA1_INTERLACED__ + 2] = _mm_set1_epi32(20); }
	if (__SHA1_INTERLACED__ > 0) { ((__m128i*) hash)[6 * __SHA1_INTERLACED__ + 0] = _mm_set1_epi32(0x80); }
	if (__SHA1_INTERLACED__ > 1) { ((__m128i*) hash)[6 * __SHA1_INTERLACED__ + 1] = _mm_set1_epi32(0x80); }
	if (__SHA1_INTERLACED__ > 2) { ((__m128i*) hash)[6 * __SHA1_INTERLACED__ + 2] = _mm_set1_epi32(0x80); }
	SHA1::calcSSE2(hash, hash);
}

#endif

void MySQLSHA1::calcNorm(vector4 *pw, vector4 *hash)
{
	SHA1::calcNorm(pw, hash + 1);
	hash[0].data[0] = 20;
	hash[6].data[0] = TO_LITTLE_ENDIAN_32(0x80);
	SHA1::calcNorm(hash, hash);
}

uint32_t MySQLSHA1::initCalc()
{
#ifdef ARC_x86
	if ((getInstructionSets() & (IS_AVX | IS_XOP)) == (IS_AVX | IS_XOP))
	{
		HashFactory::registerHash("mysqlsha1",     QUOTE(__SHA1_INTERLACED__) "xAVX_XOP", 0, 23, MySQLSHA1::calcAVXXOP,  20, 4 * __SHA1_INTERLACED__, CE_ASCII_BIN8 | CE_80_NULL_PADDED, 6, 12);
		return HashFactory::registerHash("mysql5", QUOTE(__SHA1_INTERLACED__) "xAVX_XOP", 0, 23, MySQLSHA1::calcAVXXOP,  20, 4 * __SHA1_INTERLACED__, CE_ASCII_BIN8 | CE_80_NULL_PADDED, 6, 12);
	}
	else if ((getInstructionSets() & IS_AVX) == IS_AVX)
	{
		HashFactory::registerHash("mysqlsha1",     QUOTE(__SHA1_INTERLACED__) "xAVX",     0, 23, MySQLSHA1::calcAVX,     20, 4 * __SHA1_INTERLACED__, CE_ASCII_BIN8 | CE_80_NULL_PADDED, 6, 12);
		return HashFactory::registerHash("mysql5", QUOTE(__SHA1_INTERLACED__) "xAVX",     0, 23, MySQLSHA1::calcAVX,     20, 4 * __SHA1_INTERLACED__, CE_ASCII_BIN8 | CE_80_NULL_PADDED, 6, 12);
	}
	else if ((getInstructionSets() & IS_SSSE3) == IS_SSSE3)
	{
		HashFactory::registerHash("mysqlsha1",     QUOTE(__SHA1_INTERLACED__) "xSSSE3",   0, 23, MySQLSHA1::calcSSSE3,   20, 4 * __SHA1_INTERLACED__, CE_ASCII_BIN8 | CE_80_NULL_PADDED, 6, 12);
		return HashFactory::registerHash("mysql5", QUOTE(__SHA1_INTERLACED__) "xSSSE3",   0, 23, MySQLSHA1::calcSSSE3,   20, 4 * __SHA1_INTERLACED__, CE_ASCII_BIN8 | CE_80_NULL_PADDED, 6, 12);
	}
	else if ((getInstructionSets() & IS_SSE2) == IS_SSE2)
	{
		HashFactory::registerHash("mysqlsha1",     QUOTE(__SHA1_INTERLACED__) "xSSE2",    0, 23, MySQLSHA1::calcSSE2,    20, 4 * __SHA1_INTERLACED__, CE_ASCII_BIN8 | CE_80_NULL_PADDED, 6, 12);
		return HashFactory::registerHash("mysql5", QUOTE(__SHA1_INTERLACED__) "xSSE2",    0, 23, MySQLSHA1::calcSSE2,    20, 4 * __SHA1_INTERLACED__, CE_ASCII_BIN8 | CE_80_NULL_PADDED, 6, 12);
	}
#endif
	HashFactory::registerHash("mysqlsha1",     "scalar",                              0, 23, MySQLSHA1::calcNorm,    20,                       1, CE_ASCII_BIN8 | CE_80_NULL_PADDED, 6, 2);
	return HashFactory::registerHash("mysql5", "scalar",                              0, 23, MySQLSHA1::calcNorm,    20,                       1, CE_ASCII_BIN8 | CE_80_NULL_PADDED, 6, 2);
}
