/*
	RT Bench - Benchmarks various hash functions for Rainbow Tables
	Copyright (C) 2012 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "hashfactory.h"
#include "ntlm.h"
#include "md4.h"
#include "md4help.h"

uint32_t NTLM::inited = NTLM::initCalc();

uint32_t NTLM::initCalc()
{
#ifdef ARC_x86
	if ((getInstructionSets() & (IS_AVX | IS_XOP)) == (IS_AVX | IS_XOP))
	{
		return HashFactory::registerHash("ntlm", QUOTE(__MD4_INTERLACED__) "xAVX_XOP", 0, 15, MD4::calcAVXXOP,  16, 4 * __MD4_INTERLACED__, CE_UTF16_LE | CE_80_NULL_PADDED, 8);
	}
	else if ((getInstructionSets() & IS_AVX) == IS_AVX)
	{
		return HashFactory::registerHash("ntlm", QUOTE(__MD4_INTERLACED__) "xAVX",     0, 15, MD4::calcAVX,     16, 4 * __MD4_INTERLACED__, CE_UTF16_LE | CE_80_NULL_PADDED, 8);
	}
	else if ((getInstructionSets() & IS_SSE2) == IS_SSE2)
	{
		return HashFactory::registerHash("ntlm", QUOTE(__MD4_INTERLACED__) "xSSE2",    0, 15, MD4::calcSSE2,    16, 4 * __MD4_INTERLACED__, CE_UTF16_LE | CE_80_NULL_PADDED, 8);
	}
#endif
	return HashFactory::registerHash("ntlm", "scalar",                             0, 15, MD4::calcNorm,    16,                      1, CE_UTF16_LE | CE_80_NULL_PADDED, 8);
}
