/*
	RT Bench - Benchmarks various hash functions for Rainbow Tables
	Copyright (C) 2012 Steve Thomas <steve AT tobtu DOT com>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "hashfactory.h"

list<listHash*> *HashFactory::s_hashes = NULL;

uint32_t HashFactory::getHashCharacterEncoding(string name)
{
	list<listHash*>::iterator it;

	if (HashFactory::s_hashes != NULL)
	{
		for (it = s_hashes->begin(); it != s_hashes->end(); ++it)
		{
			if ((*it)->name == name)
			{
				return (*it)->hash.charaterEncoding;
			}
		}
	}
	return 0;
}

uint32_t HashFactory::getHashInstance(string name, uint32_t minPw, uint32_t maxPw, hashAlgorithm &hash)
{
	list<listHash*>::iterator it;

	if (HashFactory::s_hashes != NULL)
	{
		for (it = s_hashes->begin(); it != s_hashes->end(); ++it)
		{
			if ((*it)->name == name && (*it)->minPw <= minPw && (*it)->maxPw >= maxPw)
			{
				hash = (*it)->hash;
				return 0;
			}
		}
	}
	hash.func             = NULL;
	hash.len              = 0;
	hash.hashesPerCall    = 0;
	hash.charaterEncoding = 0;
	return 1;
}

uint32_t HashFactory::registerHash(string name, string comment, uint32_t minPw, uint32_t maxPw, HASHFUNC hashFunc, uint32_t hashLen, uint32_t hashesPerCall, uint32_t charaterEncoding, uint32_t numPwBlocks, uint32_t numExtraHashBlocks)
{
	listHash *lh = new listHash;

	lh->hash.func               = hashFunc;
	lh->hash.len                = hashLen;
	lh->hash.hashesPerCall      = hashesPerCall;
	lh->hash.charaterEncoding   = charaterEncoding;
	lh->hash.numPwBlocks        = numPwBlocks;
	lh->hash.numExtraHashBlocks = numExtraHashBlocks;
	lh->name                    = name;
	lh->comment                 = comment;
	lh->minPw                   = minPw;
	lh->maxPw                   = maxPw;
	if (HashFactory::s_hashes == NULL)
	{
		HashFactory::s_hashes = new list<listHash*>;
	}
	s_hashes->push_back(lh);
	return 1;
}

void HashFactory::printHashes()
{
	list<listHash*>::iterator it;
	string spaces;
	uint32_t nameMaxLen = 0, commentMaxLen = 0, maxLen;

	if (HashFactory::s_hashes != NULL)
	{
		for (it = s_hashes->begin(); it != s_hashes->end(); ++it)
		{
			if (nameMaxLen < (*it)->name.length())
				nameMaxLen = (*it)->name.length();
			if (commentMaxLen < (*it)->comment.length())
				commentMaxLen = (*it)->comment.length();
		}
		maxLen = nameMaxLen > commentMaxLen ? nameMaxLen : commentMaxLen;
		spaces.reserve(maxLen);
		for (uint32_t a = 0; a < maxLen; a++)
		{
			spaces.append(" ");
		}
		for (it = s_hashes->begin(); it != s_hashes->end(); ++it)
		{
			printf(" * %s%s [%s]%s (lengths %d - %d)\n",
				(*it)->name.c_str(),    spaces.substr(0, nameMaxLen    - (*it)->name.length()).c_str(),
				(*it)->comment.c_str(), spaces.substr(0, commentMaxLen - (*it)->comment.length()).c_str(), (*it)->minPw, (*it)->maxPw);
		}
	}
	printf("\n");
}
